include (jslib_dir);
include (jslib_fileutils);
include (jslib_file);
include ("chrome://mozimage/content/mozimage_prefs.js");
include ("chrome://global/content/strres.js");

/*
**
** Private functions
**
*/

var mozImageBundle = srGetStrBundle("chrome://mozimage/locale/mozimage.properties");
var modImageLastImage = '';
var numOfLinks = 0;

setTimeout(window_load, 100);

function showFile(imageURL)
{
  var mainImage = document.getElementById("main-image");

  mainImage.removeAttribute('width');
  mainImage.removeAttribute('height');
  mainImage.src = imageURL;

  var imageBox = document.getElementById("image-box");
  if (prefs.getAutoSize())
    imageBox.setAttribute("class", "noscroll");
  else
    imageBox.setAttribute("class", "scroll");
}

function resize(ratio)
{
  var image = document.getElementById("main-image");
  var filename = image.src;
  var newWidth = image.boxObject.width * ratio;
  var newHeight = image.boxObject.height * ratio;
  
  if (prefs.getAutoSize())
    autosize_click();
  
  image.src = "";
  image.width = newWidth;
  image.height = newHeight;
  image.src = filename;
}

function autoSize()
{
  var image = document.getElementById("main-image");
  // avoid an infinite loop
  if (modImageLastImage != image.src) 
  {
    modImageLastImage = image.src;
    var newImage = new Image();
    newImage.setAttribute("id", "thepreviewimage");
    newImage.src = image.src;
    var box = document.getElementById("image-box");
    var boxWidth = box.boxObject.width;
    var boxHeight = box.boxObject.height;
    var filename = image.src;
    var xratio = newImage.width / boxWidth;
    var yratio = newImage.height / boxHeight;
    var ratio = (xratio > yratio) ? xratio : yratio;
  
    image.src = "";
    image.width = newImage.width / ratio;
    image.height = newImage.height / ratio;
    image.src = filename;
  }
}

function updateDescription()
{
  var image = document.getElementById("main-image");
  var info = document.getElementById("info-label");
  var newImage = new Image();
  newImage.setAttribute("id", "thepreviewimage");
  newImage.src = image.src;
  info.value = mozImageBundle.formatStringFromName("sizeInfo", [newImage.width, newImage.height], 2);
}

function fileNameCompare(name1,name2)
{

   if(name1.leaf.toLowerCase() < name2.leaf.toLowerCase())
      return -1;
   if(name1.leaf.toLowerCase() > name2.leaf.toLowerCase())
      return 1;
   return 0;

}

function fileNameNumCompare(name1,name2)
{
  // work with filenames such as: pic1.jpg, pic2.jpg, ..., pic9.jpg, pic10.jpg etc.
  var filenameRe = /^(\D*)(\d*)(.*)$/;
  var fn1 = name1.leaf.toLowerCase();
  var fn2 = name2.leaf.toLowerCase();
  while (fn1 != "" || fn2 != "") {
    var filenameComponents1 = filenameRe.exec(fn1);
    var filenameComponents2 = filenameRe.exec(fn2);
    if (filenameComponents1[1] < filenameComponents2[1]) return -1;
    if (filenameComponents1[1] > filenameComponents2[1]) return 1;
    var rv = filenameComponents1[2] - filenameComponents2[2];
    if (rv != 0) return rv;
    fn1 = filenameComponents1[3];
    fn2 = filenameComponents2[3];
  }
  return 0;
}

function fileCompare(name1,name2)
{
  var method = prefs.getOrderBy();
  var descending = (prefs.getDescending() ? -1 : 1);
  switch (method)
  {
    case "name" :
      return descending*fileNameCompare(name1,name2);
      break;
    case "nume" :
      return descending*fileNameNumCompare(name1,name2);
      break;
    case "date" :
      return descending*(name1.dateModified.getTime() - name2.dateModified.getTime());
      break;
    case "size" :
      return descending*(name1.size - name2.size);
      break;
    default :
      return descending*fileNameCompare(name1,name2);
  }
  return 0;
}

function fillListBox(aLocation)
{
  if (aLocation.substr(0, 7) == "http://")
    initializeSearch(aLocation);
  else
    directorySearch(aLocation);
}

function initializeSearch(url)
{
  var fullpath = document.getElementById("fullpath-text");
  var aBrowser = document.getElementById("html-browser");
  
  aBrowser.addEventListener("load", htmlSearch, true);
  
  numOfLinks = 0;
  fullpath.value = url;
  aBrowser.setAttribute('src', url);
}

function htmlSearch()
{

  var dirlistbox = document.getElementById("directory-listbox");
  var filelistbox = document.getElementById("file-listbox");
  var aBrowser = document.getElementById("html-browser");
  var doc = aBrowser.contentDocument;
  if (doc.links.length > numOfLinks)
  {
    emptyList();
    numOfLinks = doc.links.length;
    for (var i = 0 ; i < doc.links.length ; i++)
    {
      if (supportedLinkType(doc.links[i]))
        filelistbox.appendChild(getImageLinkItem(doc.links[i]));
      else if (doc.links[i].href.substr(0, 7) == "http://")
        dirlistbox.appendChild(getDirectoryItem(doc.links[i]));
    }
  }
}    

function directorySearch(adir)
{
  var dirnotfound = mozImageBundle.formatStringFromName("dirnotfound", [], 0);
  var dirlistbox = document.getElementById("directory-listbox");
  var filelistbox = document.getElementById("file-listbox");
  var d = new Dir(adir);
  var dirList = d.readDir();
  if (dirList == null)
  {
    alert(dirnotfound);
    return;
  }
  
  var fullpath = document.getElementById("fullpath-text");

  var dirs = new Array();
  var files = new Array();
  var i = 0;
  
  emptyList();
  fullpath.value = adir;
  
  for (i = 0 ; i < dirList.length ; i++) 
    dirList[i].isDir() ?  dirs.push(dirList[i]) : files.push(dirList[i]);

  dirs.sort(fileNameCompare);
  files.sort(fileCompare);

  dirList = new Array();

  for (i = 0 ; i < dirs.length ; i++)
  {
    var listItem = dirlistbox.appendItem(dirs[i].leaf, '');
    // add the attributes to show right icon
    listItem.setAttribute("class", "listitem-iconic");
    listItem.setAttribute("value", "directory");
  }

  for (i = 0 ; i < files.length ; i++)
    if (supportedType(files[i]))
      filelistbox.appendChild(getFileItem(files[i].leaf));
    
  prefs.setHomeDir(adir);
   
}

function getDirectoryItem(linkItem)
{
  var fullpath = document.getElementById("fullpath-text");
  var listItem = document.createElement('listitem');
  var listCell = document.createElement('listcell');
  var fileLabel = document.createElement('label');
  fileLabel.setAttribute('value', linkItem.href);
  fileLabel.setAttribute('crop', 'center');
  fileLabel.setAttribute('flex', '1');

  listCell.appendChild(fileLabel);
  listItem.appendChild(listCell);
  listItem.setAttribute("class", "listitem-iconic");
  listItem.setAttribute("value", "directory");
  // For compatibility with directoy viewer
  listItem.setAttribute("label", linkItem.href);
  return(listItem);
}

function getImageLinkItem(linkItem)
{
  var fullpath = document.getElementById("fullpath-text");
  var listItem = document.createElement('listitem');
  var listCell = document.createElement('listcell');
  var fileLabel = document.createElement('label');

  fileLabel.setAttribute('value', linkItem.href);
  fileLabel.setAttribute('crop', 'center');
  fileLabel.setAttribute('flex', '1');
//  fileLabel.crop = 'start';


  if (prefs.getThumbSize() > 0)
  {
    if (linkItem.firstChild.nodeName.toUpperCase() == 'IMG' || prefs.getForceHttpTumb())
    {
      var tumbURL = "";
      var fileImage = document.createElement('image');
      if (linkItem.firstChild.nodeName.toUpperCase() == 'IMG')
        tumbURL = linkItem.firstChild.getAttribute('src');
      if (tumbURL == "")
        tumbURL = linkItem.href;
      fileImage.setAttribute('src', tumbURL);
      fileImage.setAttribute('width', prefs.getThumbSize());
      fileImage.setAttribute('height', prefs.getThumbSize());
      listCell.appendChild(fileImage);
    }
  }

  listCell.appendChild(fileLabel);
  listItem.appendChild(listCell);
  return(listItem);
}

function getFileItem(fileName)
{
  var fullpath = document.getElementById("fullpath-text");
  var fileUtil = new FileUtils();
  var fullFileName = PathToURI(fileUtil.append(fullpath.value, fileName));

  var listItem = document.createElement('listitem');
  var listCell = document.createElement('listcell');
  var fileLabel = document.createElement('label');
  
  fileLabel.setAttribute('value', fileName);
  
  if (prefs.getThumbSize() > 0)
  {
    var fileImage = document.createElement('image');
    fileImage.setAttribute('src', fullFileName);
    fileImage.setAttribute('width', prefs.getThumbSize());
    fileImage.setAttribute('height', prefs.getThumbSize());
    listCell.appendChild(fileImage);
  }
  
  listCell.appendChild(fileLabel);
  listItem.appendChild(listCell);
  return(listItem);
}

function supportedLinkType(linkItem)
{
  var result = false;
  var extList = prefs.getExtList();
  for (var i = 0 ; i < extList.length ; i++)
  {
    if (extList[i].toUpperCase() == linkItem.href.substr(-3).toUpperCase())
      result = true;
  }
  return(result);
}

function supportedType(file)
{
  var result = false;
  var extList = prefs.getExtList();
  for (var i = 0 ; i < extList.length ; i++)
  {
    if (extList[i].toUpperCase() == file.ext.toUpperCase())
      result = true;
  }    
  return(result);
}

function emptyList()
{
  var dirlistbox = document.getElementById("directory-listbox");
  var filelistbox = document.getElementById("file-listbox");
  var item;
  
  var i = dirlistbox.getRowCount();
  while (i > 0)
  {
    i--;
    item = dirlistbox.getItemAtIndex(i);
    dirlistbox.removeChild(item);
  }
  i = filelistbox.getRowCount();
  while (i > 0)
  {
    i--;
    item = filelistbox.getItemAtIndex(i);
    filelistbox.removeChild(item);
  }

}

function doSlideShow()
{
  if ( prefs.getSlideshow() )
  {
    next_click();
    setTimeout(doSlideShow, prefs.getDelay());
  }
}

function selectItemByName(itemName)
{
  var fileListBox = document.getElementById("file-listbox");
  var numOfRows = fileListBox.getRowCount();
  
  for(var i = 0 ; i < numOfRows ; i++)
  {
    item = fileListBox.getItemAtIndex(i);
    fileListBox.scrollToIndex(i);
    if ( item.lastChild.lastChild.value == itemName )
    {
      fileListBox.selectItem(item);
    }

  }
  fileListBox.ensureIndexIsVisible(fileListBox.selectedIndex);
}

function parseCommandLine()
{
  var cmdLineService = Components.classes["@mozilla.org/appshell/commandLineService;1" ].getService( Components.interfaces.nsICmdLineService );
  var path = cmdLineService.URLToLoad;

  if (path != null && path != "")
  {
    var file = new File(path);
    if (file.isDir())
      fillListBox(file.path)
    else
    {
      fillListBox(file.parent.path);
      selectItemByName(file.leaf);
    }
  }
}

function buildOpenWithMenu()
{
  
  var editorsName = prefs.getEditorsName();
  for (var i = 1 ; i <= 4 ; i++)
  {
    var openwith = document.getElementById("openwith" + i + "-menu");
    openwith.label = mozImageBundle.formatStringFromName("openwith", [editorsName[i-1]], 1);
    openwith.hidden = (editorsName[i-1] == "");
  }
}

function buildMacroMenu()
{
  var macroName = prefs.getMacroName();
  for (var i = 0 ; i < 10 ; i++)
  {
    var macromenu = document.getElementById("macro" + i + "-menu");
    macromenu.label = macroName[i];
    macromenu.hidden = (macroName[i] == "");
  }
}

function PathToURI(aPath)
{
  const JS_URIFIX = new Components.Constructor("@mozilla.org/docshell/urifixup;1", "nsIURIFixup");
  var uriFixup = new JS_URIFIX();

  var fixedURI = uriFixup.createFixupURI(aPath, 0);
  return(fixedURI.spec);
}

function clearCache() 
{
  var classID = Components.classes["@mozilla.org/network/cache-service;1"];
  var cacheService = classID.getService(Components.interfaces.nsICacheService);
  cacheService.evictEntries(Components.interfaces.nsICache.STORE_ON_DISK);
  cacheService.evictEntries(Components.interfaces.nsICache.STORE_IN_MEMORY);
}

function getParams(commandLine, imagePath)
{
  var params = new Array();
  var index = 0;
  var quote = false;
  var i = 0;
  params[index] = "";
  for (i = 0 ; i < commandLine.length ; i++ )
  {
    if (commandLine[i] == "'")
      quote = !quote;
    else if (commandLine[i] == ' ' && !quote)
    {
      index++;
      params[index] = "";
    }
    else
      params[index] = params[index] + commandLine[i];
  }
  for (i = 0 ; i < params.length ; i++ )
  {
    if (params[i] == '%f')
      params[i] = imagePath;
  }
  return(params);
}

/*
**
** Event handler
**
*/

function window_load(event)
{
  var fullpath = document.getElementById("fullpath-text");
  var autoSizeMenu = document.getElementById("autosize-menu");
  var autoSizeButton = document.getElementById("autosize-button");
  var aBrowser = document.getElementById("html-browser");
  var openerLocation = "";
    
  autoSizeMenu.setAttribute("checked", prefs.getAutoSize());
  autoSizeButton.setAttribute("checked", prefs.getAutoSize());

  buildOpenWithMenu();
  buildMacroMenu();

  window.onresize = image_resize;
  
  if (window.opener != null && "gBrowser" in window.opener)
    openerLocation = window.opener.gBrowser.contentWindow.location.href;

  if (openerLocation == "")
  {
    parseCommandLine();
    if (fullpath.value == "")
      fillListBox(prefs.getHomeDir());
  }
  else
    fillListBox(openerLocation);
}

function window_close(event)
{
  var aBrowser = document.getElementById("html-browser");
  prefs.setSlideshow(false);
  prefs.save();
}

function about_click()
{
  window.openDialog('chrome://mozimage/content/mozimage_about.xul', 'about', 'chrome,centerscreen,dialog,modal');
}

function options_click()
{
  prefs.save();
  window.openDialog('chrome://mozimage/content/mozimage_prefs.xul', mozImageBundle.GetStringFromName("optionsTitle"), 'chrome,centerscreen,dialog,modal');
  prefs.load();
  buildOpenWithMenu();
  var fullpath = document.getElementById("fullpath-text");
  fillListBox(fullpath.value);
}

function macroconf_click()
{
  prefs.save();
  window.openDialog('chrome://mozimage/content/mozimage_macros.xul', 'Macros', 'chrome,centerscreen,dialog,modal');
  prefs.load();
  buildMacroMenu();
}

function exit_click()
{
  window.close();
}

function image_resize()
{
  var mainImage = document.getElementById("main-image");
  if (prefs.getAutoSize() && mainImage.src != "")
  {
    modImageLastImage = '';
    autoSize();
  }
}

function autosize_click()
{
  var mainImage = document.getElementById("main-image");
  var autoSizeMenu = document.getElementById("autosize-menu");
  var autoSizeButton = document.getElementById("autosize-button");
  var autosize = !prefs.getAutoSize();
  
  prefs.setAutoSize(autosize);
  autoSizeMenu.setAttribute('checked', autosize);
  autoSizeButton.setAttribute('checked', autosize);
  
  // Reload the image to adjust the size based on the
  // "autosize" setting
  showFile(mainImage.src);
}

function zoomout_click()
{
  resize(1/prefs.getZoom());
}

function zoomin_click()
{
  resize(prefs.getZoom());
}

function slideshow_click()
{
  var slideshowMenu = document.getElementById("slideshow-menu");
  var slideshowButton = document.getElementById("slideshow-button");
  var slideshow = !prefs.getSlideshow();
  
  prefs.setSlideshow(slideshow);
  slideshowMenu.setAttribute('checked', slideshow);
  slideshowButton.setAttribute('checked', slideshow);

  doSlideShow();
}

function image_click()
{
  var image = document.getElementById("main-image");
  image.focus();
}

function image_load()
{
  if (prefs.getAutoSize())
    autoSize();
  updateDescription();
}

function clear_click()
{
  // Clear image
  var image = document.getElementById("main-image");
  image.src = "";
  image.width = 0;
  image.height = 0;
  // and stop slide show
  var slideShowMenu = document.getElementById("slideshow-menu");
  slideShowMenu.setAttribute("checked", "false");
}

function prior_click()
{
  var fileListBox = document.getElementById("file-listbox");
  var selectedIndex = fileListBox.selectedIndex;
  if (selectedIndex >= 1)
    fileListBox.selectedIndex = selectedIndex - 1;
  else
    fileListBox.selectedIndex =  fileListBox.getRowCount() - 1;
  fileListBox.ensureIndexIsVisible ( fileListBox.selectedIndex );
}

function next_click()
{
  var fileListBox = document.getElementById("file-listbox");
  var selectedIndex = fileListBox.selectedIndex;
  if (selectedIndex >= -1 && selectedIndex < fileListBox.getRowCount() - 1)
    fileListBox.selectedIndex = selectedIndex + 1;
  else
    fileListBox.selectedIndex = 0;
  fileListBox.ensureIndexIsVisible ( fileListBox.selectedIndex );
}

function up_click()
{
  var fullpath = document.getElementById("fullpath-text");
  var fileUtil = new FileUtils();
  if (fullpath.value)
  {
    var aPath = fullpath.value;
    var file = new File(aPath);
    var aParent = file.parent.path;
    fillListBox(aParent);
  }
}

function go_click(event)
{
  var fullpath = document.getElementById("fullpath-text");
  fillListBox(fullpath.value);
}

function directory_select(event)
{
  var selectedItem = event.target.selectedItem;
  var basePath = document.getElementById("fullpath-text");
  var path = selectedItem.getAttribute("label");
  var fileUtil = new FileUtils();

  if (path.substr(0, 7) == "http://")
    fillListBox(path);
  else
    fillListBox(fileUtil.append(basePath.value, path));
}

function file_select(event)
{
  var selectedItem = event.target.selectedItem;
  var basePath = document.getElementById("fullpath-text");
  var fileUtil = new FileUtils();
  
  var labelItem = selectedItem.lastChild.lastChild;
  var fileName = labelItem.value;
  
  if (fileName.substr(0, 7) != "http://")
    var uri = PathToURI(fileUtil.append(basePath.value, fileName));
  else
    uri = fileName;
    
  showFile(uri);
}

function fullscreen_click()
{
  window.fullScreen = !window.fullScreen;

  var statusbar = document.getElementById("info-statusbar");
  var menubar = document.getElementById("menu-bar");
  var restoreButton = document.getElementById("restore-button");

  statusbar.hidden = window.fullScreen;
  menubar.hidden   = window.fullScreen;
  restoreButton.hidden = !window.fullScreen;
}

function openwith_click(index)
{
  var filelistbox = document.getElementById("file-listbox");
  var selectedItem = filelistbox.selectedItem;
  var labelItem = selectedItem.lastChild.lastChild;
  var fileName = labelItem.value;

  var basePath = document.getElementById("fullpath-text");
  var fileUtil = new FileUtils();
  var editorsPath = prefs.getEditorsPath();
  var image = document.getElementById("main-image");

  var imagePath = fileUtil.append(basePath.value, fileName);
  
  try {
    var rv = fileUtil.spawn(editorsPath[index - 1], [imagePath]);
    if (rv==null)
      alert('File not found');
  } catch (e) {
    alert(e);
  }
}

function macro_click(index)
{
  var filelistbox = document.getElementById("file-listbox");
  var selectedItem = filelistbox.selectedItem;
  var labelItem = selectedItem.lastChild.lastChild;
  var fileName = labelItem.value;
  var basePath = document.getElementById("fullpath-text");
  var fileUtil = new FileUtils();
  var imagePath = fileUtil.append(basePath.value, fileName);
  var image = document.getElementById("main-image")
  var imageURL = image.src;

  var macroCode = prefs.getMacroCode();
  var commandLine = macroCode[index];
  var paramsList = getParams(commandLine, imagePath);
  var paramsOnly = new Array();
  var command = paramsList[0];

  for (var i = 1 ; i < paramsList.length ; i++)
    paramsOnly[i-1] = paramsList[i];

  try {
    var rv = fileUtil.execute(command, paramsOnly);
    if (rv==null)
      alert('File not found');
  } catch (e) {
    alert(e);
  }
  var thumbItem = selectedItem.lastChild.firstChild;
  if (thumbItem.nodeName == "image")
    thumbItem.src = "";
  image.src = "";
  clearCache();
  image.src = imageURL;
  if (thumbItem.nodeName == "image")
    thumbItem.src = imageURL;
}

function fullpath_keypress(e)
{
  if (e.keyCode == 13)
  {
    var fullpath = document.getElementById("fullpath-text");
    fillListBox(fullpath.value);
  }
}

function togglelist_click()
{
  var fileListBox = document.getElementById("filelist-splitter");
  if (prefs.getListCollapsed())
    fileListBox.setAttribute('state', 'open');
  else
    fileListBox.setAttribute('state', 'collapsed');
  prefs.setListCollapsed( !prefs.getListCollapsed() );
}
